#include <ncurses.h>
#include <map>
#include <sys/stat.h>

#include "mls.h"
#include "MLSmain.h"
#include "MLSmcd.h"
#include "config.h"
#include "locale.h"
#include "strutil.h"
#include "keydef.h"

#include "dialog.h"

using namespace strutil;
using namespace ch;

MLSmain::MLSmain()
{
	cur=0;
	page=0;
	
	sel_size=0;
	sel_num=0;
	
	bExit=false;	
}

MLSmain::~MLSmain()
{
	// file_list 삭제..	
	pFileIterator i;
	
	for (i = file_list.begin(); i!=file_list.end(); ++i)
		delete *i;
	
	// 메모리풀도 지운다..
	for (i = memory_pool.begin(); i!=memory_pool.end(); ++i)
		delete *i;
	

}

int 
MLSmain::search(string &str)
{
	int l = file_list.size();
	
	string p=tolower(str);
	
	for (int t=0; t < l ; t++)
	{
		if ( file_list[t]->name.size() < str.size() ) continue;
		if ( tolower(file_list[t]->name.substr(0, str.size())) == p) return t;		
	}	
	// 없으면
	return -1;	
}

void 
MLSmain::proc()
{
	int key, bkey;
	bool bSearch = false;	
	
	while(!bExit)
	{
		draw();	

		key = getch();

		if (!bSearch && ('a' <= key && key <= 'z') || ('A' <= key && key <= 'Z')) bSearch = true;
		
						
		if (bSearch && ( (32<= key && key <=126) || key == KEY_BS ) )
		{
			int tmp;
			// search mode;
			// 뒷부분에 key를 넣고
			
			if (key == KEY_BS) 
			{
				strSearch.erase(strSearch.size()-1, 1);
				if (strSearch.empty()) {bSearch = false; continue;}
			}
			else strSearch += (char) key;
			
			// strSearch로 시작하는 문자를 찾는다..						
			if ((tmp = search(strSearch))!= -1) cur = tmp;
			else
			{
				strSearch = (char) key;
				// 없으면 처음에 넣고 다시 찾는다.
				
				if ((tmp = search(strSearch))!= -1) cur = tmp;
				// 없으면 말구..
				else {bSearch = false;strSearch.clear();}
			}
			continue;			
		}
		
		if (KEY_F(1) <= key && key <=KEY_F(12))
		{
			if (conf.Func[key-KEY_F(1)].size()==1)	command((conf.Func[key-KEY_F(1)])[0]);
			continue;
		}
		
		
		bSearch = false;		
		strSearch.clear();

		
		switch(key)
		{
				
		case KEY_UP:
			if (cur > 0) cur--;
			break;
			
		case KEY_DOWN:
			if (cur < file_list.size()-1) cur++;
			break;
		
		case KEY_LEFT:
			if (cur < (unsigned)nrow) cur=0;
			else cur-=nrow;
			break;
		
		case KEY_RIGHT:
			cur+=nrow;
			if (cur > file_list.size()-1) cur=file_list.size()-1;
			break;
			
		case ' ':
			if (file_list[cur]->selected) deselect(*file_list[cur]);
			else select(*file_list[cur]);
			
			if (cur < file_list.size()-1) cur++;			
			break;
			
		case '*':
			for (pFileIterator i=file_list.begin();i!=file_list.end(); i++)
			{
				if ((*i)->selected) deselect(*(*i));
				else select(*(*i));
			}
			break;

		case KEY_HOME:
			cur = 0;
			break;
			
		case KEY_END:
			cur = file_list.size()-1;
			break;
			
		
		case KEY_PPAGE:
			if (cur < (unsigned)nrow*ncol) cur=0;
			else cur-=(nrow*ncol);
			break;
			
		case KEY_NPAGE:
			cur+=(nrow*ncol);
			if (cur > file_list.size()-1) cur=file_list.size()-1;
			break;	
			
			
		case KEY_CTRL_P:
		{
			int size;
			vector<string> &p = GetSelection(size, true);
			
			for (vector<string>::iterator i=p.begin(); i!=p.end(); ++i)
				LOG("%s", (*i).c_str());
			break;
		}		
			
		case KEY_CTRL_T:
		{
			destroy();
			getch();				
			init();
			break;
		}
					
		case KEY_BS:
			read("..");
			break;
					
				
		/*case KEY_DC:
			if (!file_list[cur].bdir())
			{
				remove(file_list[cur].name.c_str());				
			}
			read(".");
			break;
		*/
		
		
		case 13:
			if (file_list[cur]->bDir())
			{
				read(file_list[cur]->name);
				break;
			}
			
			{			
			std::map<std::string, config::prog>::iterator i = conf.ext_prog.find( tolower( file_list[cur]->ext()) );
			
			if (i != conf.ext_prog.end())
			{
				if ((*i).second.size()==1)	command(((*i).second)[0]);
			}
			else if (file_list[cur]->attr[3] == 'x')
			{			
				std::string cmd;
				cmd = "./" + file_list[cur]->name;
				
				destroy();				
				system(cmd.c_str());
				init();
			}
			break;
			}
			
		case '/':
		{
			string cmds;
			if (InputBox("Shell command", cmds)<0) break;
			command(cmds);			
			break;
		}
			
		case '\\':
			read("/");
			break;
			
		case 27: //ESC-meta, alt
		
			bkey = getch();
			
			switch(bkey)
			{
			case '0':
				conf.AutoDrawmode = !conf.AutoDrawmode;				
				break;				
			case '1':
				conf.column = 1;
				conf.AutoDrawmode=false;
				break;
			case '2':
				conf.column = 2;
				conf.AutoDrawmode=false;
				break;
			case '3':
				conf.column = 3;
				conf.AutoDrawmode=false;
				break;
			case '4':
				conf.column = 4;
				conf.AutoDrawmode=false;
				break;
			case '5':
				conf.column = 5;
				conf.AutoDrawmode=false;
				break;
			case '6':
				conf.column = 6;
				conf.AutoDrawmode=false;
				break;
				
			case 'N':
			case 'n':
				conf.sort = 1;
				read(".", true);
				break;			
				
			case 'E':
			case 'e':
				conf.sort = 2;
				read(".", true);
				break;
				
			case 'S':
			case 's':
				conf.sort = 3;
				read(".", true);
				break;
				
			case 'T':
			case 't':
				conf.sort = 4;
				read(".", true);
				break;
			case 'B':
			case 'b':
				conf.sort = 5;
				read(".", true);
				break;
				
//----------------------------
			case 'C':
			case 'c':
				cmd_copy();
				break;
				
			case 'k':
			case 'K':
				command("<mkdir>");
				break;
				
			case 'R':
			case 'r':
				command("<rename>");
				break;
			
				
			case 'V':
			case 'v':			
				if (!file_list[cur]->bDir())
					command(conf.extern_viewer + " %F");
					
				break;
			
			case 'X':			
			case 'x':
				bExit=true;
				break;
				
			case 'Z':
			case 'z':			
				conf.ShowHidden = !conf.ShowHidden;
				read(".", true);
				break;
				
			case '`':
				set_locale(KO);
				break;
				
			default:			
				break;
			}
		}
	}

	string ccc = "cd " + path;

	system(ccc.c_str());
}
