#include <curses.h>
#include <vector>

#include "mls.h"
#include "MLSmain.h"
#include "config.h"
#include "locale.h"
#include "drawutil.h"
#include "strutil.h"

using namespace ch;
using namespace std;
using namespace strutil;

char kindsort[6] = {' ', 'N', 'E', 'S', 'T', 'B'};

void MLSmain::draw()
{
	int i;

	int size = file_list.size();

	if (conf.AutoDrawmode)
	{
		if (size < LINES-6) conf.column = 1;
		else if (size < (LINES-6)*2) conf.column = 2;
		else if (size < (LINES-6)*3) conf.column = 3;
		else if (size < (LINES-6)*4) conf.column = 4;
		else if (size < (LINES-6)*5) conf.column = 5;
		else conf.column = 6;
	}

	if (conf.FixedRow)  nrow=LINES-6;
	else
	{
		nrow = ((size+conf.column-1) / conf.column);
		if (nrow > LINES - 6) nrow = LINES-6;
		if (LINES - 6 < 0 ) nrow = 0;
	}

	// Function Panel
	setcol(conf.FuncN);
	for (i=1; i<9; i++)
		mvprintw(0, 2+(i-1)*10,"%-8s", conf.Func[i].getName());
	unsetcol();

	setcol(conf.FuncA);
	mvprintw (0, 0, "F2");
	for (i=0; i<7; i++)
	{
		mvprintw(0, 11+i*10, "%d", i+3);
	}
	unsetcol();

	if (conf.FuncN.back >=8 ) attron(A_BLINK);
	attron(COLOR(COLOR_BLACK, conf.FuncN.back%8));
	for (i=0; i<7;i++)
		mvaddch(0, 10+i*10, VLINE);
	unsetcol();

	// PATH
	setcol(conf.StatN);
	mvprintw(1, 0, "Path %-75s", path.c_str());
	unsetcol();


	// 파일 출력하기..
	switch(conf.column)
	{
	case 1:
		drawfile1();
		break;
	case 2:
		drawfile2();
		break;
	case 3:
		drawfile3();
		break;
	case 4:
		drawfile4();
		break;

	case 5:
		drawfile5();
		break;

	case 6:
		drawfile6();
		break;
	}


	// 찾기 창
	setcol(conf.Else);
		if (!strSearch.empty())
		mvprintw(2, 58, "[%-20s]", strSearch.c_str());
	unsetcol();

	//  FileManager Status
	setcol(conf.StatA);
	mvprintw(nrow+4, 0, "%5d ", nfile);
	mvprintw(nrow+4, 10, "%5d ", ndir);
	mvprintw(nrow+4, 19, "%14d ", dir_size);
	if (sel_num)
	{
		 mvprintw(nrow+4, 38, "%5d", sel_num);
		 if (sel_size >= 1000000000)
		 	mvprintw(nrow+4, 54, " %10.3f G", (float)sel_size/1073741824);
		 else
		 	mvprintw(nrow+4, 54, " %11.11s ", toregular(sel_size).c_str());
	}
	else mvprintw(nrow+4, 38, "%42s", "");

	unsetcol();

	setcol(conf.StatN);
	mvprintw(nrow+4, 6,"File");
	mvprintw(nrow+4, 16, "Dir");
	mvprintw(nrow+4, 34, "Byte");

	if (sel_num)
	{
		mvprintw(nrow+4, 43, " Selected (");
		mvprintw(nrow+4, 67, "Bytes)");
	}
//	mvprintw(nrow+4, 59, "byte free (2047M)    ");
	unsetcol();

	attron(COLOR(COLOR_RED, COLOR_WHITE) | A_BOLD);
	mvaddch(nrow+4, 78, kindsort[conf.sort%6]);
	unsetcol();

	// nrow + 5 ~ LINES -2

	setcol(conf.Else);
	for (i=nrow+5 ; i<=LINES-2; i++)
	{
		move(i, 0);
		printw("%-80.80s", "");
	}
	unsetcol();

	// Functional Status Panel
	// attr
	// //소유자
	// //그룹
	// 만들어진 시간
	// 크기
	// 파일 이름
	
	file &rf = *file_list[cur];

	setcol(conf.FuncN);
	mvprintw(LINES-1, 0, "%-11.11s", rf.attr);
	mvprintw(LINES-1, 12, "%10.0d", rf.size);
	mvprintw(LINES-1, 23, "%s %s", rf.date, rf.time);

	mvprintw(LINES-1, 38, "%-52.52s", rf.name.c_str());

	unsetcol();

	setcol(COLOR_BLACK, conf.FuncN.back);
	mvaddch(LINES-1, 11, VLINE);
	mvaddch(LINES-1, 22, VLINE);
	mvaddch(LINES-1, 37, VLINE);
	unsetcol();

	refresh();
}

void MLSmain::drawfile1()
{
	// 형식 1
	ncol=1;

	//  ===========================	 위아래 선 긋기
	setcol(conf.Line);
	move(2,0);	     hline(HLINE,80);
	move(nrow+3,0);  hline(HLINE,80);
	unsetcol();

	pFileIterator p = file_list.begin();

	int page = cur / (ncol * nrow);
	uint cnum=page*ncol*nrow;

	p+=(page*ncol*nrow);

	for (int row=0; row<nrow; row++)
	{
		if (p==file_list.end())
		{
			setcol(conf.Else);
			move(row+3, 0);
			printw("%-80.80s", "");
			continue;
		}
		
		file &rf = *(*p);
		
		move(row+3,0);

		if (rf.bDir())
		{
			if (cnum == cur) setrcol(conf.Dir);
			else setcol(conf.Else);
			
			printw("%s %s %s",rf.attr, rf.date, rf.time);
			unsetcol();
			
			if (cnum == cur) setrcol(conf.Dir);
			else setcol(conf.Dir);
			
			if (rf.selected) addch('*');
			else addch(' ');			
			

			if (rf.name.size() >= 42) printw("%-41.41s~ ", rf.name.c_str());
			else printw("%-42.42s ", rf.name.c_str());
			
			printw("[ SubDir ] ");

			unsetcol();

		}
		else
		{
			if (cnum == cur) setrcol(*rf.color);
			else setcol(conf.Else);

			printw("%s %s %s",rf.attr, rf.date, rf.time);
			unsetcol();
			
			if (cnum == cur) setrcol(*rf.color);
			else setcol(*rf.color);
			
			if (rf.selected) addch('*');
			else addch(' ');			

			if (rf.name.size() >= 42) printw("%-41.41s~ ", rf.name.c_str());
			else printw("%-42.42s ", rf.name.c_str());

			//printw ("%10d ", rf.size);

			if (rf.size >= 10000000)
			{
				printw("%9.1f", (float)rf.size/1048576);
				unsetcol();

				if (cnum == cur) setcol(11, rf.color->font);
				else setcol(11, conf.Else.back);
				addch('M');
				addch(' ');
				unsetcol();
			}
			else printw ("%10s ", toregular(rf.size).c_str());


			unsetcol();
		}

		p++;
		cnum++;
	}

}
/*
void MLSmain::drawfile1()
{
	// 형식 1
	ncol=1;

	//  ===========================	 위아래 선 긋기
	setcol(conf.Line);
	move(2,0);	     hline(HLINE,80);
	move(nrow+3,0);  hline(HLINE,80);
	unsetcol();

	vector<file>::iterator p=file_list.begin();

	int page = cur / (ncol * nrow);
	uint cnum=page*ncol*nrow;

	p+=(page*ncol*nrow);

	for (uint row=0; row<nrow; row++)
	{
		if (p==file_list.end())
		{
			setcol(conf.Else);
			move(row+3, 0);
			printw("%-80.80s", "");
			continue;
		}
		
		file &pf = *(*p);

		if (rf.bDir())
		{
			if (cnum == cur) setrcol(conf.Dir);
			else setcol(conf.Dir);

			if (rf.name.size() >= 42) mvprintw(row+3,0, "%-41.41s~ ", rf.name.c_str());
			else mvprintw(row+3,0, "%-42.42s ", rf.name.c_str());
			printw("[ SubDir ] ");

			unsetcol();

			if (cnum == cur) setrcol(conf.Dir);
			else setcol(conf.Else);
			printw("%s %s %s ",rf.date, rf.time, rf.attr);
			unsetcol();
		}
		else
		{
			if (cnum == cur) setrcol(rf.color);
			else setcol(rf.color);

			if (rf.name.size() >= 42) mvprintw(row+3,0, "%-41.41s~ ", rf.name.c_str());
			else mvprintw(row+3,0, "%-42.42s ", rf.name.c_str());

			unsetcol();

			if (cnum == cur) setrcol(rf.color);
			else setcol(conf.Else);

			printw ("%10d ", rf.size);
			printw("%s %s %s ",rf.date, rf.time, rf.attr);

			unsetcol();


		}

		p++;
		cnum++;
	}

}
*/

void MLSmain::drawfile2()
{
	// 형식 2
	ncol=2;

	//  ===========================	 위아래 선 긋기
	setcol(conf.Line);
	move(2,0);	     hline(HLINE,80);
	move(nrow+3,0);  hline(HLINE,80);
	unsetcol();


	for (int i=1; i<=2; i++)
	{
		move(2,i*40-1); addch(TTEE);
		for (int j=1; j<=nrow; j++)
			mvaddch (2+j, i*40-1, VLINE);
		move(nrow+3,i*40-1); addch(BTEE);
	}
	pFileIterator p=file_list.begin();

	int page = cur / (ncol * nrow);
	uint cnum=page*ncol*nrow;

	p+=(page*ncol*nrow);

	for (int col=0; col<ncol; col++)
	for (int row=0; row<nrow; row++)
	{
		if (p==file_list.end())
		{
			setcol(conf.Else);
			move(row+3, col*40);
			printw("%-39.39s", "");
			continue;
		}
		
		file &rf = *(*p);

		if (rf.bDir())
		{
			if (cnum == cur) setrcol(conf.Dir);
			else setcol(conf.Dir);

			if (rf.name.size() >= 16) mvprintw(row+3,col*40, "%-15.15s~", rf.name.c_str());
			else mvprintw(row+3,col*40, "%-16.16s", rf.name.c_str());

			if (rf.selected) addch('*');
			else addch(' ');

			printw("  DIR  ");

			unsetcol();

			if (cnum == cur) setrcol(conf.Dir);
			else setcol(conf.Else);


			printw("%s %s ",rf.date, rf.time);
			unsetcol();
		}
		else
		{
			if (cnum == cur) setrcol(*rf.color);
			else setcol(*rf.color);

			if (rf.name.size() >= 16) mvprintw(row+3,col*40, "%-15.15s~", rf.name.c_str());
			else mvprintw(row+3,col*40, "%-16.16s", rf.name.c_str());

			if (rf.selected) addch('*');
			else addch(' ');

			unsetcol();

			if (cnum == cur) setrcol(*rf.color);
			else setcol(conf.Else);


			if (rf.size >= 1000000)
			{
				printw("%5.1f", (float)rf.size/1048576);
				unsetcol();

				if (cnum == cur) setcol(11, rf.color->font);
				else setcol(11, conf.Else.back);
				addch('M');

				unsetcol();

				if (cnum == cur) setrcol(*rf.color);
				else setcol(conf.Else);


			}
			else printw ("%6d", rf.size);
			printw(" %s %s ",rf.date, rf.time);

			unsetcol();

		}

		p++;
		cnum++;
	}

}

void MLSmain::drawfile3()
{
	ncol=3;
	//  ===========================	 위아래 선 긋기

	setcol(conf.Line);
	move(2,0);	     hline(HLINE,80);
	move(nrow+3,0);  hline(HLINE,80);
	unsetcol();

	for (int i=1; i<=3; i++)
	{
		move(2,i*27-2); addch(TTEE);
		for (int j=1; j<=nrow; j++)
			mvaddch (2+j, i*27-2, VLINE);
		move(nrow+3,i*27-2); addch(BTEE);
	}

	unsetcol();

	// 형식 3
	pFileIterator p=file_list.begin();
	int page = cur / (ncol * nrow);
	uint cnum=page*ncol*nrow;

	p+=(page*ncol*nrow);

	for (int col=0; col<ncol; col++)
		for (int row=0; row<nrow; row++)
		{
			if (p==file_list.end())
			{
				setcol(conf.Else);
				if (col!=0)
				{
					move(row+3, col*27-1);
					printw("%-26.26s", "");
				}
				else
				{
					move(row+3, 0);
					printw("%-25.25s", "");
				}
				continue;
			}
			
			file &rf = *(*p);

			if (rf.bDir())
			{
				if (cnum == cur) setrcol(conf.Dir);
				else setcol(conf.Dir);
				if (col == 0) move(row+3, 0);
				else {move(row+3, col*27-1); addch(' ');}

				if (rf.name.size() > 24) printw("%-23.23s~", rf.name.c_str());
				else printw("%-24s", rf.name.c_str());
				
				if (rf.selected) addch('*');
				else addch(' ');
				

				unsetcol();
			}
			else
			{
				if (cnum == cur) setrcol(*rf.color);
				else setcol(*rf.color);

				if (col == 0) move(row+3, 0);
				else {move(row+3, col*27-1); addch(' ');}

				if (rf.name.size() > 17) printw("%-16.16s~ ", rf.name.c_str());
				else printw("%-17s ", rf.name.c_str());

				unsetcol();

				if (cnum == cur) setrcol(*rf.color);
				else setcol(conf.Else);

				if (rf.size >= 1000000)
				{
					printw("%5.1f", (float)rf.size/1048576);
					unsetcol();
					if (cnum == cur) setcol(11, rf.color->font);
					else setcol(11, conf.Else.back);
					addch('M');
					unsetcol();
				}
				else printw("%6d", rf.size);
				unsetcol();
				
				if (cnum == cur) setrcol(*rf.color);
				else setcol(*rf.color);
				
				
				if (rf.selected) addch('*');
				else addch(' ');
				
				unsetcol();
			}

			p++;
			cnum++;
		}
	return;
}


void MLSmain::drawfile4()
{
	ncol=4;
	//  ===========================	 위아래 선 긋기

	setcol(conf.Line);
	move(2,0);	     hline(HLINE,80);
	move(nrow+3,0);  hline(HLINE,80);
	unsetcol();


	for (int i=1; i<=4; i++)
	{
		move(2,i*20-1); addch(TTEE);
		for (int j=1; j<=nrow; j++)
			mvaddch (2+j, i*20-1, VLINE);
		move(nrow+3,i*20-1); addch(BTEE);
	}

	unsetcol();

	// 형식 4
	pFileIterator p=file_list.begin();

	int page = cur / (ncol * nrow);
	uint cnum=page*ncol*nrow;

	p+=(page*ncol*nrow);
	for (int col=0; col<ncol; col++)
		for (int row=0; row<nrow; row++)
		{
			if (p==file_list.end())
			{
				setcol(conf.Else);
				move(row+3, col*20);
				printw("%-19.19s", "");
				continue;
			}
			
			file &rf = *(*p);

			if (rf.bDir())
			{
				if (cnum == cur) setrcol(conf.Dir);
				else setcol(conf.Dir);
				move(row+3, col*20);
				//if (rf.name.size() > 12) printw("%-11.11s~   DIR ", rf.name.c_str());
				//else printw("%-12s   DIR ", rf.name.c_str());

				if (rf.name.size() > 18) printw("%-17.17s~", rf.name.c_str());
				else printw("%-18s", rf.name.c_str());
				
				if (rf.selected) addch('*');
				else addch(' ');				

				unsetcol();
			}
			else
			{
				move(row+3, col*20);

				if (cnum == cur) setrcol(*rf.color);
				else setcol(*rf.color);

				if (rf.name.size() > 12) printw("%-11.11s~ ", rf.name.c_str());
				else printw("%-12s ", rf.name.c_str());

				unsetcol();

				if (cnum == cur) setrcol(*rf.color);
				else setcol(conf.Else);
				if (rf.size >= 100000)
				{
					printw("%5.1f", (float)rf.size/1048576);
					unsetcol();

					if (cnum == cur) setcol(11, rf.color->font);
					else setcol(11, conf.Else.back);
					addch('M');
				}
				else printw("%5d", rf.size);
				unsetcol();
				
				if (cnum == cur) setrcol(*rf.color);
				else setcol(*rf.color);				
				
				if (rf.selected) addch('*');
				else addch(' ');
				
				unsetcol();
			}

			p++;
			cnum++;
		}
	return;
}

void MLSmain::drawfile5()
{
	ncol=5;
	//  ===========================	 위아래 선 긋기

	setcol(conf.Line);
	move(2,0);	     hline(HLINE,80);
	move(nrow+3,0);  hline(HLINE,80);
	unsetcol();

	for (int i=1; i<=5; i++)
	{
		move(2,i*16-1); addch(TTEE);
		for (int j=1; j<=nrow; j++)
			mvaddch (2+j, i*16-1, VLINE);
		move(nrow+3,i*16-1); addch(BTEE);
	}

	unsetcol();

	// 형식 5
	pFileIterator p=file_list.begin();

	int page = cur / (ncol * nrow);
	uint cnum=page*ncol*nrow;

	p+=(page*ncol*nrow);
	for (int col=0; col<ncol; col++)
		for (int row=0; row<nrow; row++)
		{
			if (p==file_list.end())
			{
				setcol(conf.Else);
				move(row+3, col*16);
				printw("%-15.15s", "");	
				unsetcol();
				continue;
			}
			
			file &rf = *(*p);
					
			move(row+3, col*16);
			
			if (rf.bDir())
			{
				if (cnum == cur) setrcol(conf.Dir);
				else setcol(conf.Dir);
			}
			else
			{
				if (cnum == cur) setrcol(*rf.color);
				else setcol(*rf.color);
			}
			
			if (rf.selected) addch('*');
			else addch(' ');			
			
			if (rf.name.size() > 14) printw("%-13.13s~", rf.name.c_str());
			else printw("%-14s", rf.name.c_str());

			unsetcol();
			
			p++;
			cnum++;
		}
	return;
}



void MLSmain::drawfile6()
{
	ncol=6;
	//  ===========================	 위아래 선 긋기

	setcol(conf.Line);
	move(2,0);	     hline(HLINE,80);
	move(nrow+3,0);  hline(HLINE,80);
	unsetcol();

	for (int i=1; i<=5; i++)
	{
		move(2,i*13); addch(TTEE);
		for (int j=1; j<=nrow; j++)
			mvaddch (2+j, i*13, VLINE);
		move(nrow+3,i*13); addch(BTEE);
	}

	unsetcol();

	// 형식 6
	pFileIterator p=file_list.begin();

	int page = cur / (ncol * nrow);
	uint cnum=page*ncol*nrow;

	p+=(page*ncol*nrow);
	for (int col=0; col<ncol; col++)
		for (int row=0; row<nrow; row++)
		{
			if (p==file_list.end())
			{
				setcol(conf.Else);

				if (col == 0)
				{
					move(row+3, 0);
					printw("%-13.13s", "");
				}
				else if (col == 5)
				{
					move(row+3, 66);
					printw("%-14.14s", "");
				}
				else
				{
					move(row+3, col*13+1);
					printw("%-12.12s", "");
				}
				unsetcol();

				continue;
			}
			file &rf = *(*p);

			setcol(conf.Else);
			if (col == 0) {move(row+3, 0); addch(' ');}
			else move(row+3, col*13+1);
			unsetcol();

			if (rf.bDir())
			{
				if (cnum == cur) setrcol(conf.Dir);
				else setcol(conf.Dir);
			}
			else
			{
				if (cnum == cur) setrcol(*rf.color);
				else setcol(*rf.color);	
			}
			
			if (rf.selected) addch('*');
			else addch(' ');

			if (rf.name.size() > 11) printw("%-10.10s~", rf.name.c_str());
			else printw("%-11s", rf.name.c_str());

			unsetcol();	

			if (col == 5)
			{
				setcol(conf.Else);
				addch (' ');
				addch (' ');
				unsetcol();
			}

			p++;
			cnum++;
		}
	return;
}

