#include <dirent.h>
#include <sys/stat.h>
#include <time.h>
#include <string>
#include <map>

#include <unistd.h>

#include "MLSmain.h"
#include "config.h"
#include "strutil.h"
#include "dialog.h"

using namespace std;
using namespace strutil;

int MLSmain::read(string dir_name, bool preserve)
{
	DIR        *handle;
	dirent     *pFile;
	struct stat stat_bufs;
	
	if (chdir(dir_name.c_str())==-1)
		return -1;
		
	if ((handle = opendir("."))==NULL) return -1;
		
	string        cur_name;		
	if (preserve) cur_name = file_list[cur]->name;		
		
	cur      = 0;
	page     = 0;
	sel_size = 0;
	sel_num  = 0;	

	nfile    = 0;
	ndir     = 0;
	dir_size = 0;
	
	
	string     oldpath = path;
	
	char *tmpdir  = getcwd(NULL, 0);
	bool  bRoot   = !strcmp(tmpdir, "/");	
	path = tmpdir;
	free(tmpdir);
	
	if (path != "/") path += '/';
	
	bool  mem_ovr = true;
	file *t=NULL;	
	
	pFileIterator fi;	
		
	if (!file_list.empty()) fi = file_list.begin();
	else mem_ovr = false;
	
	
	while((pFile = readdir(handle)))
	{		
		if (!strcmp(pFile->d_name, "."))	continue;		
		if (bRoot && !strcmp(pFile->d_name, "..")) continue;
		
		if (!conf.ShowHidden && pFile->d_name[0] == '.' && strcmp(pFile->d_name, "..") ) continue;
		
		// 파일 정보얻어오기
		if (stat(pFile->d_name, &stat_bufs)==-1) continue;		
		
		if (mem_ovr) t = *fi;		
		else 
		{
			if (memory_pool.empty()) t = new file;
			else
			{
				t = memory_pool.back();
				memory_pool.pop_back();
			}
		}
		
		t->name     = pFile->d_name;		
		t->selected = false;
		
		// attr		
		t->mode = stat_bufs.st_mode;
		
		if (S_ISDIR (stat_bufs.st_mode)) {t->attr[0]='d'; ndir++;}
		else {t->attr[0]='.'; nfile++;}		
		t->attr[1]= S_IRUSR & stat_bufs.st_mode ? 'r' : '.';
		t->attr[2]= S_IWUSR & stat_bufs.st_mode ? 'w' : '.';
		t->attr[3]= S_IXUSR & stat_bufs.st_mode ? 'x' : '.';		
		t->attr[4]= S_IRGRP & stat_bufs.st_mode ? 'r' : '.';
		t->attr[5]= S_IWGRP & stat_bufs.st_mode ? 'w' : '.';
		t->attr[6]= S_IXGRP & stat_bufs.st_mode ? 'x' : '.';		
		t->attr[7]= S_IROTH & stat_bufs.st_mode ? 'r' : '.';
		t->attr[8]= S_IWOTH & stat_bufs.st_mode ? 'w' : '.';
		t->attr[9]= S_IXOTH & stat_bufs.st_mode ? 'x' : '.';		
		t->attr[10]='\0';		
		
		// ext를 구해보자.
		map<string, color_entry>::iterator exti = conf.ext_col.find( tolower( t->ext()) );		
		if (exti != conf.ext_col.end() ) 
			t->color = &(*exti).second;
		else 
			t->color = &conf.Else;
		
		t->size = stat_bufs.st_size;
		t->mtime = stat_bufs.st_mtime;		
		strftime(t->date, 9, conf.date_type.c_str(), localtime(&stat_bufs.st_mtime));
		strftime(t->time, 6, "%H:%M", localtime(&stat_bufs.st_mtime));		
		
		if (mem_ovr) 
		{
			++fi;
			if (fi == file_list.end()) mem_ovr = false;
		}
		else file_list.push_back(t);
		
		dir_size += t->size;
	}
	closedir(handle);
	
	// 남는 메모리들은 메모리 풀로...
	if (mem_ovr)
	{
		copy(fi, file_list.end(), back_inserter(memory_pool));
		file_list.erase(fi, file_list.end());
	}
	
	// 소팅을 시작합니다	
	if (file_list.size() > 1)
	{	
		pFileIterator i, 
			 begin = file_list.begin(),
			 end   = file_list.end();
		
		if (conf.DirSort)
		{
			if (!bRoot) 
			{// ..을 찾아서 맨 앞으로(루트는 없다)
				for(i = begin; i != end; i++)
					if ((*i)->name == "..")								
					{
						file *tmp = *begin;
						*begin = *i;
						*i = tmp;
						begin++;
						break;
					}
			}
			
			// 디렉토리들을 찾아서 앞으로..
			pFileIterator s=end-1;
			
			for (;begin!=end; begin++)
			{
				if (!(*begin)->bDir())
				{
					while(s != begin && !(*s)->bDir()) s--;
					
					if (begin == s) break;
					
					file *tmp   = *begin;
					     *begin = *s;
					     *s     = tmp;	
				}
			}
			
			if (bRoot) sort(file_list.begin(), begin, sort_name());
			else sort(file_list.begin()+1, begin, sort_name());
		}	
		
		// 정렬	
		switch(conf.sort)
		{
		case 1: // name
			sort(begin, end, sort_name());	
			break;
		case 2: // ext
			sort(begin, end, sort_ext());	
			break;
		case 3: // size
			sort(begin, end, sort_size());	
			break;	
		case 4: // time
			sort(begin, end, sort_time());	
			break;
		case 5:  // color
			sort(begin, end, sort_color());	
			break;			
		case 0:
		default:
			break;
		}
		
	}
	
	// 커서보존..
	if (preserve)
	{	
		pFileIterator i;
		for (i=file_list.begin(); i!=file_list.end(); ++i)
			if ((*i)->name == cur_name) break;
				
		if (i!=file_list.end()) cur = i - file_list.begin();		
	}
	else if (dir_name == "..") // ..에 해당하는 디렉토리를 찾는다..
	{
		oldpath.erase(oldpath.size()-1, 1);
		
		uint t = oldpath.rfind("/", oldpath.size());	
		const char *p      = oldpath.c_str()+t+1;
		
		for (pFileIterator i=file_list.begin(); i!=file_list.end(); i++)
		{
			if (!(*i)->bDir()) continue;
			
			if ((*i)->name==p)
			{
				cur = i-file_list.begin();
				break;
			}
		}		
	}
	

	return 1;
}
