#ifndef MLSmcd_H__
#define MLSmcd_H__

#include <string>
#include <vector>
#include <unistd.h>

#include "mls.h"

class MLSmcd
{
protected:

	struct dir
	{
		std::string name;
		int  		depth, row, index;		
		struct dir  *parent;
		bool        linked;
		std::vector<dir*> node;
		
		dir(std::string p, dir *pa, bool l = false) 
			: name(p), parent(pa), linked(l) {depth = pa ? pa->depth+1 : 0;}
		
		std::string &path();
		
	};
	
	typedef std::vector<dir *>	         pDirContainer;
	typedef std::vector<dir *>::iterator pDirIterator;		
	
	pDirContainer order;  		// 순회 테이블
	pDirIterator  cur;
	
	dir           *root;        // 루트 트리..  마지막 '/' 을 뺀 이름만을 기록	

	int           sRow, sCol;   // 스크롤 한 Row와 Col	
	bool          bExit;
	
	std::string   infofile;		// treeinfo 파일 위치
	char          *code;        // treeinfo 헤더
	const char          *title;
	
	// 순회테이블 재정리, cur 무효화
	void setOrder();
	
	// 커서이동	
	void left();
	void right();
	void up();
	void down();
	
	// scan fuction	
	int  load();
	int  save();
	int  rescan();	
	int  scan(dir *);
	void destroy(dir *, bool = true);	
	
	// draw function
	void draw();

public:	
	MLSmcd(const char * = "MCD - Mls Change Directory");
	~MLSmcd();
	
	void setCur(std::string);
	std::string &getCur() {return (*cur)->path();}
	
	bool proc();
};

#endif //MLSmcd_H__
