#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <cassert>

#include "config.h"
#include "MLSmcd.h"

using namespace std;

int MLSmcd::save()
{
	FILE *fp=fopen(infofile.c_str(), "wb");
	
	fwrite(code, 4, 1, fp);
	
	for (pDirIterator i = order.begin(); i!= order.end(); i++)
	{	
		fputc ((*i)->depth, fp);
		if ((*i)->linked) fputc (1, fp);
		else fputc(0, fp);		
		fputc ((*i)->name.size() / 256, fp);
		fputc ((*i)->name.size() % 256, fp);
		fputs ((*i)->name.c_str(), fp);
	}
	
	fclose(fp);
	
	return 0;
}

int MLSmcd::load()
{
	dir *pNode=NULL;
	dir  *preNode;
	int      size, depth, preDepth=0, row=0;
	FILE    *fp;
	char    *name;
	char     head[5];
	bool     linked;
	
	fp = fopen(infofile.c_str(), "rb");
		
	if (!fp) return 1;
	
	fgets(head, 5, fp);	
	if (strcmp(head, code)) return 1;
	
	while(!feof(fp))
	{
		depth=fgetc(fp);
		
		if (fgetc(fp)) linked = true;
		else linked = false;
		
		size =fgetc(fp)*256;
		size +=fgetc(fp);
		
		if (feof(fp)) break;
		
		name = new char[size+1];
		fread(name, size, 1, fp);
		name[size] = '\0';	
		
		// depth에 따른 노드 조정
		if (depth > preDepth) 
		{
			pNode = preNode;
		}
		else
		{
			while(depth < preDepth)
			{
				pNode = pNode->parent;
				preDepth--;
			}
			if (pNode) row++;
		}
	
		preNode = new dir(name ,pNode, linked);
		preNode->row = row;
		
		delete name;		
		
		if (pNode) pNode->node.push_back(preNode);
		else pNode = preNode;
		
		preDepth = depth;
	}
	
	fclose(fp);
	
	while(pNode->parent) pNode=pNode->parent;	
	root=pNode;

	setOrder();	
	return 0;
}

// tree 이하의 노드를 모두 지운다.
void MLSmcd::destroy(dir *tree, bool bOrdering)
{// 비재귀	
	if (tree==NULL) return;
	if (order.empty()) return;
	
	int depth = tree->depth;	
	
	for (pDirIterator i=order.begin()+tree->index+1; i!=order.end(); i++)
	{
		if (depth >= (*i)->depth) break;
		delete *i;
	}
	tree->node.clear();
	
	if (bOrdering) 
	{
		setOrder();
		save();
	}
	
	/* 재귀판
	if (!tree) return;
	for (pDirIterator i=tree->node.begin(); i!=tree->node.end(); i++)
		destroy((*i));
	
	delete tree;
	tree=NULL;*/
}

// tree로 넘어온 디렉토리에 서브디렉토리를 추가한다 
int MLSmcd::scan(dir *arg)
{// 비재귀판
	pDirContainer st;
	MLSmcd::dir  *tree;		
	DIR          *handle;
	dirent     	 *pFile;
	struct stat   stat_bufs;
	char *lpath = new char[FILENAME_MAX]; // links 용
	char *curdir = getcwd(NULL, 0);
	
	int n;
	
	try
	{	
	destroy(arg, false);	// 구성요소들을 destroy.. 시킨다..
	nodelay(stdscr, TRUE);
		
	st.push_back(arg);	
	
	while(!st.empty())
	{
		tree = st.back();
		st.pop_back();		
		
		if (tree->linked) continue;
		
		if (getch()==27) break;		
						
		if (::chdir(tree->path().c_str())==-1) continue;
		
		if ((handle = opendir("."))==NULL) continue;
		
		
		while((pFile = readdir(handle)))
		{
			if (!strcmp(pFile->d_name, "."))	continue;
			if (!strcmp(pFile->d_name, ".."))	continue;
			
			// 파일 정보얻어오기
			if (lstat(pFile->d_name, &stat_bufs)==-1) continue;
			if (S_ISLNK(stat_bufs.st_mode))
			{				
				if ((n = readlink(pFile->d_name, lpath, FILENAME_MAX)) == -1) continue;
				lpath[n] = '\0';
				
				if (lstat(lpath, &stat_bufs) == -1) continue;
				if (S_ISDIR(stat_bufs.st_mode))
					tree->node.push_back(new dir(pFile->d_name, tree, true));
				continue;				
			}			
			if (!S_ISDIR (stat_bufs.st_mode)) continue;			
			
			tree->node.push_back(new dir(pFile->d_name, tree));
		}
		closedir(handle);
				
		// 스택에 노드들을 채운다.. 순서는 반대로 한다.
		copy (tree->node.rbegin(), tree->node.rend(), back_inserter(st));		
	}
	
	nodelay(stdscr, FALSE);
	setOrder();
	
	//재스캔 했으면 저장한다
	save();
	
	}
	catch(...)
	{
		delete []lpath;
		free(curdir);
		throw;
	}	
	chdir(curdir);
	
	delete []lpath;
	free(curdir);	
	
	return 0;
}

void 
MLSmcd::setOrder()
{
	vector<dir*> st;
	dir *node;
	int ind=0;
	int row=0, odepth=-1;
	
	order.clear();
			
	st.push_back(root);	
	while(!st.empty())
	{
		node = st.back();		
		st.pop_back();
		
		if (node->depth <= odepth) row++;
		odepth = node->depth;
		
		node->row  = row;				
		node->index= ind;
		
		ind++;
		order.push_back(node);				
		copy(node->node.rbegin(), node->node.rend(), back_inserter(st));
	}
	cur = order.begin();
}


int MLSmcd::rescan()
{
	if (root) 
	{
		for (pDirIterator p = order.begin(); p!=order.end(); ++p)
			delete *p;
	}
	root = new dir(conf.mcdroot, NULL);
	return scan(root); // 루트를 중심으로 다시 스캔	
}
