#include <map>
#include <pwd.h>
#include <unistd.h>
#include <sys/stat.h>

#include "mls.h"
#include "config.h"
#include "strutil.h"

using namespace std;
using namespace strutil;

config conf;

config::config()
{
	set_default();
	
	string cfgfile = cfghome + "mls.cfg";	
		
	if (load(cfgfile.c_str())==1) 
	{
		printf ("loading mls.cfg file is failed ");
		exit(1);
	}
	
	colorset = cfghome + colorset;
	
	if (load_colorset(colorset.c_str())==1)
	{
		fprintf (stderr, "There is no colorset file");
		exit(1);
	}
	
	// 후처리
	
	if (mcdroot.empty()) mcdroot = '/';	
	else if (mcdroot[mcdroot.size()-1] != '/') mcdroot += '/';	
}

config::~config()
{
}

void config::set_default()
{// default를 만듭니다..
	struct stat 	  stat_bufs;	
	// home 디렉토리 지정
	struct passwd *pw = getpwuid(getuid()); // getuid이냐 geteuid냐...;
	home  = pw->pw_dir;
	home += '/';
	
	cfghome = home + ".mls/";
	
	if (lstat (cfghome.c_str(), &stat_bufs) == -1)
	{
		fprintf (stderr, "MLS : making mls configuration directory...");
		if (mkdir(cfghome.c_str(), 0700) == -1) 
		{
			fprintf (stderr, "MLS : making directory operation failed...");
			exit(-1);
		}		
	}
	
	mcdroot = home;
	
	// 각종 설정
	ShowHidden   = false;
	AutoDrawmode = true;
	FixedRow     = false;	
	sort         = 1;
	column       = 1;
	colorset     = "mls.col";
}


void config::prog::addEntry(string &p)
{
	//	line 안에서 cmd랑 name을 분리한다..
	//  초반의 [] 는 name
	
	// [name] prog1; prog2; prog3; ...;;
	
	if (p =="") return;
	
	if (p[0] == '[')
		name = getbetween (p, '[', ']');
		
	string::size_type os = p.find_first_of(']', p.find_first_of('[') );
	string::size_type s  = p.find_first_of(';');
	
	while (s!=string::npos)
	{	
		string t = chop(p.substr(os+1, s-os-1));
		if (t!="") addCommand(t);
		
		os=s;
		s=p.find_first_of(';', s);
	}
	string t=chop(p.substr(os+1));
	if (t!="") addCommand(t);
} 

int config::load(const char *file)
{
	char line[1024];	
	string::size_type p;
	
	map<string, string> cmd;
	string l, variable, value;
		
	FILE *fp=fopen(file, "rb");
	
	if (!fp) return 1;
	
	while(!feof(fp))
	{
		line[0]=0;
		fgets(line, 1024, fp);
						
		if (line[0] == '#') continue;
		
		l=line;
		
		// = 을 찾습니다.
		p = l.find('=');		
				
		if (p == string::npos) continue;
		
		variable = chop(l.substr(0, p));
		value =  chop(l.substr(p+1));
		
		if (value == "") continue;
		
		if (variable.substr(0, 3) == "ext")
		{
			string exts=getbetween(variable, '[', ']');		
			
			if (exts != "")
			{
				string::size_type s=exts.find_first_of(';'), os=0;
				
				prog pr;
				pr.addEntry(value);			
						
				while (s!=string::npos)
				{
					string t=chop(exts.substr(os, s-os));
					if (t != "") ext_prog[tolower(t)] = pr;
					
					os=s+1;
					s=exts.find_first_of(';', s+1);				
				}
				string t=chop(exts.substr(os));
				if (t!="") ext_prog[tolower(t)] = pr;
				
				continue;			
			}
		}				
		cmd[tolower(variable)]= value; // 나머지들..
	}
	
	fclose(fp);
	
	map<string,string>::iterator i, end = cmd.end();	
	
	// 펑션키	
	if ((i = cmd.find("funcf1")) != end) Func[0].addEntry((*i).second);
	if ((i = cmd.find("funcf2")) != end) Func[1].addEntry((*i).second);
	if ((i = cmd.find("funcf3")) != end) Func[2].addEntry((*i).second);
	if ((i = cmd.find("funcf4")) != end) Func[3].addEntry((*i).second);
	if ((i = cmd.find("funcf5")) != end) Func[4].addEntry((*i).second);
	if ((i = cmd.find("funcf6")) != end) Func[5].addEntry((*i).second);
	if ((i = cmd.find("funcf7")) != end) Func[6].addEntry((*i).second);
	if ((i = cmd.find("funcf8")) != end) Func[7].addEntry((*i).second);
	if ((i = cmd.find("funcf9")) != end) Func[8].addEntry((*i).second);
	if ((i = cmd.find("funcf10"))!= end) Func[9].addEntry((*i).second);
	if ((i = cmd.find("funcf11"))!= end) Func[10].addEntry((*i).second);
	if ((i = cmd.find("funcf12"))!= end) Func[11].addEntry((*i).second);
	
	// 문자형	
	if ((i = cmd.find("colorset")) != end )  colorset = (*i).second;
	if ((i = cmd.find("date_type")) != end ) date_type = (*i).second;
	if ((i = cmd.find("extviewer")) != end ) extern_viewer = (*i).second;
	if ((i = cmd.find("exteditor")) != end ) extern_editor = (*i).second;	
	if ((i = cmd.find("mcdroot")) != end ) mcdroot = (*i).second;	
	
	// On/Off 형	
	if ((i = cmd.find("fixedrow")) != end ) 
		FixedRow = (tolower((*i).second) == "on");
	if ((i = cmd.find("dirsort")) != end )
		DirSort  = (tolower((*i).second) == "on");
	if ((i = cmd.find("showhidden")) != end )
		ShowHidden  = (tolower((*i).second) == "on");
		
	// 정수형	
	if ((i = cmd.find("sort")) != end )   sort = atoi((*i).second.c_str());
	if ((i = cmd.find("column")) != end ) column = atoi((*i).second.c_str());	
	
	AutoDrawmode = column ? false : true;
	
	return 0;
}

int config::load_colorset(const char *file)
{
	char line[1024], *p, *loline;
	map<string, color_entry> color_set;	
	
	FILE *fp=fopen(file, "rb");
	
	if (!fp) return 1;
	
	
	while(!feof(fp))
	{
		line[0]=0;
		fgets(line, 1024, fp);
				
		if (line[0] == '#') continue;
				
		// = 을 찾습니다.
		p = strchr (line, '=');
				
		if (!p) continue;
		// 이전까지를 토큰으로 간주해서 chop 시킵니다.
		// 앞쪽 chop
		*p=0;
		
		loline=tolower(chop(line));
		
		// ext[]		
		// [ 와 ]를 찾습니다.
		
		char *sp=strchr(line, '['), *ep;
		
					
		if (sp)
		{			
			*sp=0;			
			
			if (!strcmp(loline, "ext"))
			{//
			
				ep=strchr(sp+1, ']');
				*ep=0;
				tolower(sp+1);
				
				color_entry entry(chop(p+1));		
				char *exts=strtok(sp+1, ";");				
								
				while(exts)
				{				
					ext_col[exts]=entry;
					exts=strtok(NULL, ";");					
				}				
				continue;			
			}
			*sp='[';
		}
					
		// 이외의 것들은.. colorset입니다.				
		color_set[loline]= color_entry(chop(p+1));		
	}

	fclose(fp);
	
	MCD = color_set["mcd"];
	MCDstat = color_set["mcdstat"];
	Dir = color_set["dir"];
	Drive = color_set["drive"];
	Line = color_set["line"]; 
	FuncN = color_set["funcn"];
	FuncA = color_set["funca"];
	StatN = color_set["statn"];
	StatA = color_set["stata"];
	Else = color_set["else"];
	Help = color_set["help"];
	Quick = color_set["quick"];
	QuickMCD = color_set["quickmcd"];
	Dialog = color_set["dialog"];
	
	return 0;
	
}
