#include <ncurses.h>
#include <string.h>
#include <stdarg.h>

#include "dialog.h"
#include "locale.h"
#include "drawutil.h"
#include "keydef.h"

using namespace ch;

int MsgBox(int b_c, const char *title, const char *msg,...)
{
	va_list args;
    char szBuf[256];

    va_start(args,msg);
    vsprintf(szBuf,msg,args);
    va_end(args);
    
    return MsgBox(title, szBuf, b_c);
}


int MsgBox(const char *title, const char *msg, int back_color)
{
	int l_title = strlen(title);
	int l_msg = strlen(msg);
	int width = l_title > l_msg ? l_title : l_msg;
	
	width +=4;
			
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
	
	wattron(win ,A_BOLD);
	box(win, VLINE, HLINE);
	
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	mvwprintw(win, 1, (width - l_title)/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	// msg
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	mvwprintw(win, 3, (width - l_msg)/2, "%s", msg);
	wattroff(win, A_BOLD);
	
	wrefresh(win);
	
	getch();
	
	delwin(win);		
	return 1;	
}

int InputBox(const char *title, std::string &str, int back_color)
{ // 글자가 넘어갈 경우 스크롤
	int l_title = strlen(title);	
	int width = COLS - 4;
	
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
		
	wattron(win ,A_BOLD);
	box(win, VLINE, HLINE);	
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	mvwprintw(win, 1, (width - l_title)/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	int key;
	
	while(1)
	{
		wattron (win, COLOR(COLOR_WHITE, back_color));
		wattron (win, A_BOLD);
		mvwprintw(win, 3, 2, "%-72.72s", str.c_str());
		wattroff(win, A_BOLD);
		wmove(win, 3, 2+str.size());
		
		wrefresh(win);
		
		key = getch();
		
		if (key == 27) return -1;
		if (key == 13) break;
		if (key == 263) //backspace
		{
			if (!str.empty())	str.erase(str.size()-1, 1);
			continue;
		}
				
		str += (char)key;		
	}
	delwin(win);
	return 1;	
}

int YNBox(const char *title, int YN, int back_color)
{
	int l_title = strlen(title);
	int width = l_title > 17 ? l_title : 17;	

	width +=4;
			
	WINDOW *win = newwin(5, width, (LINES-5)/2, (COLS-width)/2);
	
	wbkgd(win, COLOR(COLOR_WHITE, back_color));
	
	wattron(win ,A_BOLD);
	box(win, VLINE, HLINE);
	
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', width-2);
	mvwprintw(win, 1, (width - l_title)/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	int key;
	
	//--Yes--- ---No---
	while(1)
	{// 출력
		wmove (win, 3, (width-17)/2);
		
		if (YN == YN_Y)
			wattron (win, COLOR(back_color, COLOR_WHITE));
		else 
		{
			wattron (win, COLOR(COLOR_WHITE, back_color));
			wattron (win, A_BOLD);
		}
		wprintw(win, "  Yes   ");	
		wattroff(win, A_BOLD);
		
		wattron (win, COLOR(COLOR_WHITE, back_color));
		wattron (win, A_BOLD);		
		waddch(win, ' ');
		wattroff(win, A_BOLD);
		
		if (YN == YN_N)
			wattron (win, COLOR(back_color, COLOR_WHITE));
		else 
		{
			wattron (win, COLOR(COLOR_WHITE, back_color));
			wattron (win, A_BOLD);
		}
									
		wprintw(win,"   No   ");
		wattroff(win, A_BOLD);		
		
		wrefresh(win);		
		
		key=getch();
		
		if (key == 13) break;
		
		if (key == 27)
		{
			delwin(win);
			return -1;
		}
		
		switch(key)
		{
		case KEY_UP:
		case KEY_DOWN:
		case KEY_LEFT:
		case KEY_RIGHT:
			if (YN == YN_Y) YN=YN_N;
			else YN = YN_Y;
			break;		
		}						
		
	}	
	delwin(win);
	
	return 	YN;
}

StatBox::StatBox(const char *_title, const char *_msg, int _back_color)
{
	title = _title;
	msg =_msg;
	back_color = _back_color;
	
	UL = NULL;	
	UR = NULL;
	BL = NULL;
	BR = NULL;
	
	U=0;
	B=0;	
	
	win = newwin(12, 72, (LINES-12)/2, (COLS-72)/2);
	redraw();
	
}

StatBox::~StatBox()
{
	delwin(win);
}

void StatBox::redraw()
{
	wbkgd(win, COLOR(COLOR_WHITE, back_color));	
	
	wattron(win ,A_BOLD);
	box(win, VLINE, HLINE);	
	wattroff(win, A_BOLD);
	
	// title 출력
	wattron(win, COLOR(COLOR_BLACK, COLOR_WHITE));
	wmove(win, 1, 1);
	whline(win, ' ', 72-2);
	mvwprintw(win, 1, (72 - strlen(title))/2 , "%s", title);
	wattroff(win, A_BOLD);
	
	// msg
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	mvwprintw(win, 3, (72 - strlen(msg))/2, "%s", msg);
	wattroff(win, A_BOLD);	
}

//-
//타이틀
//
//To c:
//
//|  Autoexec
//----...
//
//<21/132>
//----...
//
//---

void StatBox::show()
{
	wattron (win, COLOR(COLOR_WHITE, back_color));
	wattron (win, A_BOLD);
	// UL UR 출력
	
	wmove(win, 5, 6);
	whline(win, ' ', 60);
	if (UL) mvwprintw(win, 5, 6, "%s", UL);
	if (UR) mvwprintw(win, 5, 72 - strlen(UR) -6, "%s", UR);	
	
	// BL BR 출력
	wmove(win, 8, 6);
	whline(win, ' ', 60);	
	if (BL) mvwprintw(win, 8, 6, "%s", BL);
	if (BR) mvwprintw(win, 8, 72 - strlen(BR) -6, "%s", BR);	
	
	wattroff(win, A_BOLD);	
	
	//U, B 기본바탕
	wattron(win, COLOR(COLOR_WHITE, COLOR_BLACK));	
	wattron(win, A_BOLD);
	wmove(win, 6, 6);
	whline(win, '.', 60);	
	wmove(win, 9, 6);
	whline(win, '.', 60);	
	wattroff(win, A_BOLD);
	
	// U, B 기본이 아닌 바탕
	wattron(win, COLOR(COLOR_WHITE, COLOR_WHITE));
	
	wmove(win, 6, 6);
	whline(win, ' ', U);
	wmove(win, 9, 6);
	whline(win, ' ', B);
	
	wrefresh(win);
}
